package model;

/*
 * Requirement:
 * You are required to implement all methods recursively.
 */
public class RecursiveMethods {
	/*
	 * We have triangle made of blocks. 
	 * The topmost row has 1 block, the next row down has 2 blocks, the next row has 3 blocks, and so on. 
	 * Compute recursively (no loops or multiplication) the total number of blocks in such a triangle with the given number of rows.
	 * 
	 * triangle(0) → 0
	 * triangle(1) → 1
	 * triangle(2) → 3
	 * 
	 * Assumption: 'rows' is larger than or equal to zero.
	 * Hint: Visually, how do the example input triangles look like? 
	 */
	public int triangle(int rows) {
		if(rows == 0) {
			return 0;
		}
		else { /* rows > 0 */
			return rows + triangle(rows - 1);
		}
	}

	/*
	 * Given an array of integers, compute recursively if the array contains somewhere a value followed in the array by that value times 10. We'll use the convention of considering only the part of the array that begins at the given index. In this way, a recursive call can pass index+1 to move down the array. 
	 * The initial call will pass in index as 0.
	 * 
	 * array220([1, 2, 20], 0) → true
	 * array220([3, 30], 0) → true
	 * array220([3], 0) → false
	 * 
	 * Try two versions: 
	 * 	boolean array220(int[] nums, int from)
	 * 	boolean array220(int[] nums, int from, int to)
	 * 		Version 1 assumes that the ‘to’ index denotes the end of the array.
	 * 		Version 2 does not make such an assumption, though ‘to’ typically is the last index.
	 * 
	 * Hint: Max value of ‘from’ before an ArrayIndexOutOfBoundsException occurs?
	 */
	public boolean array220(int[] nums, int from) {
		if(from >= nums.length - 1) {
			/*
			 * nums.length - 1 denotes the last valid index in 'nums'
			 * from > nums.length - 1 -> beyond the last index (e.g., 0 when nums is empty)
			 * from == nums.length - 1 -> last index (no more followed element to consider)
			 * 
			 * Having reached to these base cases means that all previous elements
			 * that we looked at do not satisfy the property.
			 */
			return false; 
		}
		else {
			return
				nums[from] * 10 == nums[from + 1]
				||
				array220(nums, from + 1);
		}
	}

	public boolean array220(int[] nums, int from, int to) {
		if(from >= to) {
			/*
			 * from > to -> empty range to look at
			 * from == to -> range of one element to look at
			 */
			return false;
		}
		else {
			return
					nums[from] * 10 == nums[from + 1]
					||
					array220(nums, from + 1, to);
		}
	}


	/*
	 * Return an array storing the first n numbers in an arithmetic sequence,
	 * with initial term 'start' and common difference 'diff'.  
	 * You can assume that n is non-negative (larger than or equal to 0).
	 * e.g., arithmeticArray(2, 3, 5) returns an array {2, 5, 8, 11, 14}.
	 * 
	 * Parameters:
	 * `start`: the first term in an arithmetic sequence
	 * `diff`: the common difference between terms in an arithmetic sequence
	 * `n`: the first n numbers in an arithmetic sequence
	 * 
	 *  You are forbidden to use the arithmeticList method below to solve this problem.
	 * 
	 * Requirement:
	 * You are required to implement all methods recursively.
	 * You receive a zero if there is any occurrence of a loop (e.g., for, while). 
	 */
	public int[] arithmeticArray(int start, int diff, int n) {
		if(n == 0) {
			int[] seq = new int[0];
			return seq;
		}
		else if(n == 1) {
			int[] seq = new int[1]; // int[] seq = {start};
			seq[0] = start;
			return seq;
		}
		else {
			int[] seq = new int[n];
			seq[0] = start;
			arithmeticArrayHelper(1, diff, seq);
			return seq;
		}
	}

	/*
	 * This is a recursive helper method expected to be used by arithmeticArray.
	 * 
	 * Parameters:
	 * `i`: position in `seq` to be assigned to
	 * `diff`: common difference of the arithmetics sequence 
	 * `seq`: a partially filled arithmetic sequence
	 *  
	 * Each recursive call to this helper method stores at index `i` of the resulting
	 * arithmetic sequence `seq`, which is assumed to have been partially filled at indices 0, 1, ..., i - 1. 
	 */
	private void arithmeticArrayHelper(int i, int diff, int[] seq) {
		if(i < seq.length) { // only fill out the array `seq` when `i` is not too large
			seq[i] = seq[i - 1] + diff;
			arithmeticArrayHelper(i + 1, diff, seq);
		}
	}
}
